/*
TempBan-Plugin
by Sir D. 21.07.2007
- stores time bans in adminmod/config/temp_banned.cfg
- will recreate time bans by recalculating remaining 
bantime after server crash when banned player connect
- command: admin_tempban <"player name"> <bantime>
- command: admin_tempban_list: displays a list of temp_banned.cfg

Hints: 
- file_access_read and file_access_write must set to 1 in adminmod.cfg
- ACCESS_BAN is needed to execute command by admin
- IT CAN'T be used on LAN. It only stores STEAM_IDs
(maybe change line 120 and 163 the word "bBanByID" with "bBanByIP" get working it for LAN)
- Only players currently playing on server can be banned.

Changelog:
0.92: 
	- new command admin_tempban_list to see temp_banned.cfg
0.91: 
	- new messages in chat area for player and admins
	- changed command admin_tempban to <"player name"> <bantime>
0.9: 
	- data is loaded in RAM and checked on players connect for temp bans
0.8: 
	- data stored in temp file and checked file for temp bans in this file
	- BUG: remaining 59 seconds time ban gives players a permanent ban! :(	
*/

#include <string>
#include <admin>
#include <adminlib>

new STRING_VERSION[MAX_TEXT_LENGTH]="AM 2.50.60_0.92";
new Userbancfg[MAX_TEXT_LENGTH]="addons/adminmod/config/temp_banned.cfg";
new g_Specmode[MAX_PLAYERS];

#define MAX_ENTRIES	30

new g_AuthID[MAX_ENTRIES][MAX_AUTHID_LENGTH];
new g_FileTimeStamp[MAX_ENTRIES];
new g_BanTime[MAX_ENTRIES];

new g_loaded;

public plugin_init() {
	plugin_registerinfo("TempBan-Plugin","Bans player temporarly and stores data in temp_banned.cfg",STRING_VERSION);
	plugin_registercmd("admin_tempban","admin_tempban",ACCESS_BAN,"admin_tempban <playername> <bantime>: bans player for x minutes");
	plugin_registercmd("admin_tempban_list","admin_tempban_list",ACCESS_BAN,"admin_tempban_list: displays current temp ban list");
	plugin_registercmd("specmode","specmode",ACCESS_ALL);
	load_file(0);
	return PLUGIN_CONTINUE;
}

public admin_tempban_list(HLCommand,HLData,HLUserName,UserIndex) {
	load_file(1);
	return PLUGIN_HANDLED;
}

load_file(look){
	if(fileexists(Userbancfg)){
		new Data[MAX_DATA_LENGTH];
		new Info[MAX_DATA_LENGTH];
		new FileTimeStamp[MAX_NUMBER_LENGTH];
		new AuthID[MAX_AUTHID_LENGTH];
		new BanTime[MAX_NUMBER_LENGTH];
		new iFileSize;
		new i;
		new filetimestamp;
		new bantime;
		
		if(look){
			selfmessage("[PLUGIN TEMP-BAN-LIST]");
		}
		
		iFileSize=filesize(Userbancfg);
		for(i=1;i<=iFileSize;i++){
			readfile(Userbancfg,Data,i,MAX_DATA_LENGTH);
			if(look){
				strsubst(Data,"","-",MAX_DATA_LENGTH);
				selfmessage(Data);
			}else{
				if(g_loaded<MAX_ENTRIES){
					if(strncmp(Data,"//",2)!=0){
						strsep(Data,"",FileTimeStamp,MAX_NUMBER_LENGTH,AuthID,MAX_AUTHID_LENGTH,BanTime,MAX_NUMBER_LENGTH,Info,MAX_TEXT_LENGTH);

						//get old timestamp of file as int
						filetimestamp=strtonum(FileTimeStamp);
						//Calculate new bantime in seconds as int
						bantime=strtonum(BanTime)*60-(systemtime()-filetimestamp);

						if(bantime<=0){
							//comment out line, he can play again
							snprintf(Data,MAX_DATA_LENGTH,"//%s",Data);
							writefile(Userbancfg,Data,i);
						}else{
							//store in RAM to make a faster check for STEAM_ID, if temp banned player connects after server crash
							strcpy(g_AuthID[g_loaded],AuthID,MAX_AUTHID_LENGTH);
							//as int
							g_FileTimeStamp[g_loaded]=filetimestamp;
							//as int, back to minutes
							g_BanTime[g_loaded]=bantime/60;
							g_loaded++;
						}
					}
				}else{
					selfmessage("[PLUGIN TEMP-BAN] Max. of 30 entries reached. Please check plugin_temp_ban.sma and increase MAX_ENTRIES. Recompile plugin afterwards.");
				}
			}
		}
		if(!look){
			g_loaded--;
		}
		snprintf(Data,MAX_DATA_LENGTH,"[PLUGIN TEMP-BAN] Currently %i temp bans are loaded!",g_loaded+1);
		selfmessage(Data);
	}
}


public plugin_disconnect(HLUserName, UserIndex){
	g_Specmode[UserIndex]=0;
	return PLUGIN_CONTINUE;
}

//Player enters game
public specmode(HLCommand,HLData,HLUserName,UserIndex) {
	//only check, if g_loaded >= 0 -> temp ban exists.
	if(!g_Specmode[UserIndex] && g_loaded!=-1){
		new Data[MAX_DATA_LENGTH];
		new User[MAX_NAME_LENGTH];
		new AuthID[MAX_AUTHID_LENGTH];
		convert_string(HLData,Data,MAX_DATA_LENGTH);
		convert_string(HLUserName,User,MAX_DATA_LENGTH);
		get_userAuthID(User,AuthID);
		//if server has crashed check if time ban still exists for player's steam_id 
		check_tempban(AuthID,User);
		g_Specmode[UserIndex]=1;
	}
	return PLUGIN_CONTINUE;
}

check_tempban(AuthID[],User[]){
	new Data[MAX_DATA_LENGTH];
	new i;
	for(i=0;i<=g_loaded;i++){
		if(strcmp(g_AuthID[i],AuthID)==0 && g_BanTime[i]>0){
			//ups... player still has a time ban... lets ban him again for rest of minutes.
			snprintf(Data,MAX_DATA_LENGTH,"Admin: [TEMP BAN exists for %s: %i min till unban]",User,g_BanTime[i]);
			say(Data);
			messageex(User,Data,print_console);
			ban(User,g_BanTime[i],bBanByID);
			break;
		}
	}
}

public admin_tempban(HLCommand,HLData,HLUserName,UserIndex) {
	new Data[MAX_DATA_LENGTH];
	new User[MAX_NAME_LENGTH];
	new Player[MAX_NAME_LENGTH];
	new TargetName[MAX_NAME_LENGTH];
	new BanTime[MAX_NUMBER_LENGTH];
	new AuthID[MAX_AUTHID_LENGTH];
	new timestamp;
	new bantime;
	
	convert_string(HLData,Data,MAX_DATA_LENGTH);
	convert_string(HLUserName,User,MAX_DATA_LENGTH);
	
	strgsep(Data," ","^"",Player,MAX_NAME_LENGTH,BanTime,MAX_NUMBER_LENGTH);
	/* no bantime, do nothing */
	bantime=strtonum(BanTime);
	if(!bantime){
		selfmessage("[PLUGIN TEMP-BAN]");
		selfmessage("No bantime found. Maybe use quotes around playername that bantime can be identified properly!");
		return PLUGIN_HANDLED;
	}
	if (check_user(Player)) {
		get_username(Player,TargetName,MAX_NAME_LENGTH);
		if(check_immunity(TargetName)) {
			snprintf(Data, MAX_TEXT_LENGTH, "Laf. You can't ban '%s', you silly bear.", TargetName);
			selfmessage(Data);
			return PLUGIN_HANDLED;
		}
		
		/*write it to file*/
		timestamp=systemtime();
		get_userAuthID(TargetName,AuthID);
		snprintf(Data,MAX_TEXT_LENGTH,"%i%s%i%sAdmin:%s",timestamp,AuthID,bantime,TargetName,User);
		writefile(Userbancfg,Data,-1);
		
		/*store it in RAM*/
		if(g_loaded<MAX_ENTRIES){
			g_loaded++;
			strcpy(g_AuthID[g_loaded],AuthID,MAX_AUTHID_LENGTH);
			g_FileTimeStamp[g_loaded]=timestamp;
			g_BanTime[g_loaded]=bantime;
		}
		
		/*--- Messages for admin and player ---*/
		snprintf(Data,MAX_TEXT_LENGTH,"[PLUGIN TEMP-BAN] %s has been banned by %s for %i minutes!",TargetName,User,bantime);
		say(Data);
		messageex(TargetName,Data,print_console);
		ban(TargetName,bantime,bBanByID);
	}else{
		selfmessage("[PLUGIN TEMP-BAN]");
		selfmessage("Player not identified. Please be more specific and/or use quotes around playername!");
	}
	return PLUGIN_HANDLED;
}
